from __future__ import annotations
import ngsolve as ngsolve
from ngsolve.comp import BilinearForm
from ngsolve.comp import FESpace
from ngsolve.comp import L2
from ngsolve.comp import LinearForm
from ngsolve.comp import SymbolicBFI
from ngsolve.comp import SymbolicLFI
from ngsolve.fem import CoordCF
from ngsolve.fem import ET
from ngsolve.fem import IntegrationRule
import pyngcore.pyngcore
import typing
__all__ = ['BilinearForm', 'BoxDifferentialSymbol', 'BoxIntegral', 'ClipCoefficientFunction', 'CompoundEmbTrefftzFESpace', 'CondenseDG', 'CoordCF', 'ET', 'EmbeddedTrefftzFES', 'FESpace', 'FFacetBFI', 'FFacetLFI', 'GetWave', 'IntegrationPointFunction', 'IntegrationRule', 'L2', 'L2EmbTrefftzFESpace', 'LinearForm', 'Mesh1dTents', 'MonomialEmbTrefftzFESpace', 'PUFESpace', 'PrintCF', 'QTWaveTents1', 'QTWaveTents2', 'SpaceTimeDG_FFacetBFI', 'SpaceTimeDG_FFacetLFI', 'SymbolicBFI', 'SymbolicLFI', 'TWave', 'TWaveTents1', 'TWaveTents2', 'TWaveTents3', 'Tent', 'TentSlab', 'TrefftzEmbedding', 'TrefftzTents', 'VectorL2EmbTrefftzFESpace', 'WeightedRadiusFunction', 'dbox', 'monomialfespace', 'ngsolve', 'ngstrefftz', 'trefftzfespace']
class BoxDifferentialSymbol(ngsolve.comp.DifferentialSymbol):
    """
    
    dBox that allows to formulate linear, bilinear forms and integrals on
    (bounding) boxes
    
    Example use case:
    
      dbox = BoxDifferentialSymbol()
      a = BilinearForm(...)
      a += u * v * dbox(element_boundary=...)
    
    """
    def __call__(self, definedon: ngsolve.comp.Region | str | None = None, element_boundary: bool = False, element_vb: ngsolve.comp.VorB = ..., deformation: ngsolve.comp.GridFunction = None, definedonelements: pyngcore.pyngcore.BitArray = None, bonus_intorder: int = 0, reference_box_length: float = 0.5) -> BoxDifferentialSymbol:
        """
        The call of a BoxDifferentialSymbol allows to specify what is needed to specify the
        integration domain. It returns a new BoxDifferentialSymbol.
        
        Parameters:
        
        definedon (Region or Array) : specifies on which part of the mesh (in terms of regions)
          the current form shall be defined.
        element_boundary (bool) : Does the integral take place on the boundary of an element-
          boundary?
        element_vb (VOL/BND) : Where does the integral take place from point of view
          of an element (BBND/BBBND are not implemented).
        deformation (GridFunction) : which mesh deformation shall be applied (default : None)
        definedonelements (BitArray) : Set of elements or facets where the integral shall be
          defined.
        bonus_intorder (int) : additional integration order for the integration rule (default: 0)
        reference_box_length (double) : length of the reference box (default: 0.5)
        """
    def __init__(self) -> None:
        """
        Constructor of BoxDifferentialSymbol.
        
          Argument: none
        """
    @property
    def element_vb(self) -> ngsolve.comp.VorB:
        """
        box volume or box boundary integral on each (volume) element?
        """
    @element_vb.setter
    def element_vb(self, arg1: ngsolve.comp.VorB) -> ngsolve.comp.VorB:
        ...
class BoxIntegral(ngsolve.comp.Integral):
    """
    
            BoxIntegral allows to formulate linear, bilinear forms and integrals on
            box parts of the mesh"
    """
class CompoundEmbTrefftzFESpace(ngsolve.comp.FESpace):
    """
    
    
    
    Keyword arguments can be:
    
    order: int = 1
      order of finite element space
    complex: bool = False
      Set if FESpace should be complex
    dirichlet: regexpr
      Regular expression string defining the dirichlet boundary.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet = 'top|right'
    dirichlet_bbnd: regexpr
      Regular expression string defining the dirichlet bboundary,
      i.e. points in 2D and edges in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbnd = 'top|right'
    dirichlet_bbbnd: regexpr
      Regular expression string defining the dirichlet bbboundary,
      i.e. points in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbbnd = 'top|right'
    definedon: Region or regexpr
      FESpace is only defined on specific Region, created with mesh.Materials('regexpr')
      or mesh.Boundaries('regexpr'). If given a regexpr, the region is assumed to be
      mesh.Materials('regexpr').
    dim: int = 1
      Create multi dimensional FESpace (i.e. [H1]^3)
    dgjumps: bool = False
      Enable discontinuous space for DG methods, this flag is needed for DG methods,
      since the dofs have a different coupling then and this changes the sparsity
      pattern of matrices.
    autoupdate: bool = False
      Automatically update on a change to the mesh.
    low_order_space: bool = True
      Generate a lowest order space together with the high-order space,
      needed for some preconditioners.
    hoprolongation: bool = False
      Create high order prolongation operators,
      only available for H1 and L2 on simplicial meshes
    order_policy: ORDER_POLICY = ORDER_POLICY.OLDSTYLE
      CONSTANT .. use the same fixed order for all elements,
      NODAL ..... use the same order for nodes of same shape,
      VARIABLE ... use an individual order for each edge, face and cell,
      OLDSTYLE .. as it used to be for the last decade
    print: bool = False
      (historic) print some output into file set by 'SetTestoutFile'
    """
    @staticmethod
    def __flags_doc__() -> dict:
        ...
    def Embed(self, arg0: ngsolve.comp.GridFunction) -> ngsolve.comp.GridFunction:
        ...
    def GetEmbedding(self) -> BaseMatrix:
        ...
    @typing.overload
    def SetOp(self, bf: ngsolve.comp.SumOfIntegrals, lf: ngsolve.comp.SumOfIntegrals = None, eps: float = 0, test_fes: ngsolve.comp.FESpace = None, tndof: int = 0) -> ngsolve.la.BaseVector:
        """
        Sets the operators for the embedded Trefftz method.
        """
    @typing.overload
    def SetOp(self, op: ngsolve.comp.SumOfIntegrals | None, cop_lhs: ngsolve.comp.SumOfIntegrals | None, cop_rhs: ngsolve.comp.SumOfIntegrals | None, fes_conformity: ngsolve.comp.FESpace, fes_test: ngsolve.comp.FESpace = None, linear_form: ngsolve.comp.SumOfIntegrals = None, ndof_trefftz: int | float = 0) -> ngsolve.la.BaseVector:
        """
                    Sets the operators for the conforming Trefftz method.
        
                    :param op: the differential operation. Can be None
                    :param cop_lhs: left hand side of the conformity operation
                    :param cop_rhs: right hand side of the conformity operation
                    :param fes_conformity: finite element space of the conformity operation
                    :param fes_test: test finite element space for `op`. Can be None
                    :param linear_form: right hand side of the var. formulation. Can be None
                    :param ndof_trefftz: number of degrees of freedom per element
                        in the Trefftz finite element space on `fes`, generated by `op`
                        (i.e. the local dimension of the kernel of `op` on one element)
        
                    :return: the particular solution vector.
        """
    def __getstate__(self: ngsolve.comp.FESpace) -> tuple:
        ...
    @typing.overload
    def __init__(self, mesh: ngsolve.comp.Mesh, **kwargs) -> None:
        ...
    @typing.overload
    def __init__(self, fes: ngsolve.comp.ProductSpace) -> None:
        ...
    def __setstate__(self, arg0: tuple) -> None:
        ...
class IntegrationPointFunction(ngsolve.fem.CoefficientFunction):
    def Export(self) -> list[list[float]]:
        ...
    def PrintTable(self) -> None:
        ...
    @typing.overload
    def __init__(self, mesh: ngsolve.comp.Mesh, intrule: ngsolve.fem.IntegrationRule, Vector: ngsolve.bla.VectorD) -> None:
        ...
    @typing.overload
    def __init__(self, mesh: ngsolve.comp.Mesh, intrule: ngsolve.fem.IntegrationRule, Matrix: ngsolve.bla.MatrixD) -> None:
        ...
class L2EmbTrefftzFESpace(ngsolve.comp.FESpace):
    """
    An L2-conforming finite element space.
    
    The L2 finite element space consists of element-wise polynomials,
    which are discontinuous from element to element. It uses an
    L2-orthogonal hierarchical basis which leads to orthogonal
    mass-matrices on non-curved elements.
    
    Boundary values are not meaningful for an L2 function space.
    
    The L2 space supports element-wise variable order, which can be set
    for ELEMENT-nodes.
    
    Per default, all dofs are local dofs and are condensed if static
    condensation is performed. The lowest order can be kept in the
    WIRE_BASKET via the flag 'lowest_order_wb=True'.
    
    All dofs can be hidden. Then the basis functions don't show up in the
    global system.
    
    Keyword arguments can be:
    
    order: int = 1
      order of finite element space
    complex: bool = False
      Set if FESpace should be complex
    dirichlet: regexpr
      Regular expression string defining the dirichlet boundary.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet = 'top|right'
    dirichlet_bbnd: regexpr
      Regular expression string defining the dirichlet bboundary,
      i.e. points in 2D and edges in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbnd = 'top|right'
    dirichlet_bbbnd: regexpr
      Regular expression string defining the dirichlet bbboundary,
      i.e. points in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbbnd = 'top|right'
    definedon: Region or regexpr
      FESpace is only defined on specific Region, created with mesh.Materials('regexpr')
      or mesh.Boundaries('regexpr'). If given a regexpr, the region is assumed to be
      mesh.Materials('regexpr').
    dim: int = 1
      Create multi dimensional FESpace (i.e. [H1]^3)
    dgjumps: bool = False
      Enable discontinuous space for DG methods, this flag is needed for DG methods,
      since the dofs have a different coupling then and this changes the sparsity
      pattern of matrices.
    autoupdate: bool = False
      Automatically update on a change to the mesh.
    low_order_space: bool = True
      Generate a lowest order space together with the high-order space,
      needed for some preconditioners.
    hoprolongation: bool = False
      Create high order prolongation operators,
      only available for H1 and L2 on simplicial meshes
    order_policy: ORDER_POLICY = ORDER_POLICY.OLDSTYLE
      CONSTANT .. use the same fixed order for all elements,
      NODAL ..... use the same order for nodes of same shape,
      VARIABLE ... use an individual order for each edge, face and cell,
      OLDSTYLE .. as it used to be for the last decade
    print: bool = False
      (historic) print some output into file set by 'SetTestoutFile'
    all_dofs_together: bool = True
      Change ordering of dofs. If this flag ist set,
      all dofs of an element are ordered successively.
      Otherwise, the lowest order dofs (the constants)
      of all elements are ordered first.
    lowest_order_wb: bool = False
      Keep lowest order dof in WIRE_BASKET
    hide_all_dofs: bool = False
      Set all used dofs to HIDDEN_DOFs
    tp: bool = False
      Use sum-factorization for evaluation
    """
    @staticmethod
    def __flags_doc__() -> dict:
        ...
    def Embed(self, arg0: ngsolve.comp.GridFunction) -> ngsolve.comp.GridFunction:
        ...
    def GetEmbedding(self) -> BaseMatrix:
        ...
    @typing.overload
    def SetOp(self, bf: ngsolve.comp.SumOfIntegrals, lf: ngsolve.comp.SumOfIntegrals = None, eps: float = 0, test_fes: ngsolve.comp.FESpace = None, tndof: int = 0) -> ngsolve.la.BaseVector:
        """
        Sets the operators for the embedded Trefftz method.
        """
    @typing.overload
    def SetOp(self, op: ngsolve.comp.SumOfIntegrals | None, cop_lhs: ngsolve.comp.SumOfIntegrals | None, cop_rhs: ngsolve.comp.SumOfIntegrals | None, fes_conformity: ngsolve.comp.FESpace, fes_test: ngsolve.comp.FESpace = None, linear_form: ngsolve.comp.SumOfIntegrals = None, ndof_trefftz: int | float = 0) -> ngsolve.la.BaseVector:
        """
                    Sets the operators for the conforming Trefftz method.
        
                    :param op: the differential operation. Can be None
                    :param cop_lhs: left hand side of the conformity operation
                    :param cop_rhs: right hand side of the conformity operation
                    :param fes_conformity: finite element space of the conformity operation
                    :param fes_test: test finite element space for `op`. Can be None
                    :param linear_form: right hand side of the var. formulation. Can be None
                    :param ndof_trefftz: number of degrees of freedom per element
                        in the Trefftz finite element space on `fes`, generated by `op`
                        (i.e. the local dimension of the kernel of `op` on one element)
        
                    :return: the particular solution vector.
        """
    def __getstate__(self: ngsolve.comp.FESpace) -> tuple:
        ...
    @typing.overload
    def __init__(self, mesh: ngsolve.comp.Mesh, **kwargs) -> None:
        ...
    @typing.overload
    def __init__(self, fes: ngsolve.comp.L2) -> None:
        ...
    def __setstate__(self, arg0: tuple) -> None:
        ...
class MonomialEmbTrefftzFESpace(ngsolve.comp.FESpace):
    """
    
    
    
    Keyword arguments can be:
    
    order: int = 1
      order of finite element space
    complex: bool = False
      Set if FESpace should be complex
    dirichlet: regexpr
      Regular expression string defining the dirichlet boundary.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet = 'top|right'
    dirichlet_bbnd: regexpr
      Regular expression string defining the dirichlet bboundary,
      i.e. points in 2D and edges in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbnd = 'top|right'
    dirichlet_bbbnd: regexpr
      Regular expression string defining the dirichlet bbboundary,
      i.e. points in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbbnd = 'top|right'
    definedon: Region or regexpr
      FESpace is only defined on specific Region, created with mesh.Materials('regexpr')
      or mesh.Boundaries('regexpr'). If given a regexpr, the region is assumed to be
      mesh.Materials('regexpr').
    dim: int = 1
      Create multi dimensional FESpace (i.e. [H1]^3)
    dgjumps: bool = False
      Enable discontinuous space for DG methods, this flag is needed for DG methods,
      since the dofs have a different coupling then and this changes the sparsity
      pattern of matrices.
    autoupdate: bool = False
      Automatically update on a change to the mesh.
    low_order_space: bool = True
      Generate a lowest order space together with the high-order space,
      needed for some preconditioners.
    hoprolongation: bool = False
      Create high order prolongation operators,
      only available for H1 and L2 on simplicial meshes
    order_policy: ORDER_POLICY = ORDER_POLICY.OLDSTYLE
      CONSTANT .. use the same fixed order for all elements,
      NODAL ..... use the same order for nodes of same shape,
      VARIABLE ... use an individual order for each edge, face and cell,
      OLDSTYLE .. as it used to be for the last decade
    print: bool = False
      (historic) print some output into file set by 'SetTestoutFile'
    useshift: bool = True
      shift of basis functins to element center
    usescale: bool = True
      scale element basis functions with diam
    """
    @staticmethod
    def __flags_doc__() -> dict:
        ...
    def Embed(self, arg0: ngsolve.comp.GridFunction) -> ngsolve.comp.GridFunction:
        ...
    def GetEmbedding(self) -> BaseMatrix:
        ...
    @typing.overload
    def SetOp(self, bf: ngsolve.comp.SumOfIntegrals, lf: ngsolve.comp.SumOfIntegrals = None, eps: float = 0, test_fes: ngsolve.comp.FESpace = None, tndof: int = 0) -> ngsolve.la.BaseVector:
        """
        Sets the operators for the embedded Trefftz method.
        """
    @typing.overload
    def SetOp(self, op: ngsolve.comp.SumOfIntegrals | None, cop_lhs: ngsolve.comp.SumOfIntegrals | None, cop_rhs: ngsolve.comp.SumOfIntegrals | None, fes_conformity: ngsolve.comp.FESpace, fes_test: ngsolve.comp.FESpace = None, linear_form: ngsolve.comp.SumOfIntegrals = None, ndof_trefftz: int | float = 0) -> ngsolve.la.BaseVector:
        """
                    Sets the operators for the conforming Trefftz method.
        
                    :param op: the differential operation. Can be None
                    :param cop_lhs: left hand side of the conformity operation
                    :param cop_rhs: right hand side of the conformity operation
                    :param fes_conformity: finite element space of the conformity operation
                    :param fes_test: test finite element space for `op`. Can be None
                    :param linear_form: right hand side of the var. formulation. Can be None
                    :param ndof_trefftz: number of degrees of freedom per element
                        in the Trefftz finite element space on `fes`, generated by `op`
                        (i.e. the local dimension of the kernel of `op` on one element)
        
                    :return: the particular solution vector.
        """
    def __getstate__(self: ngsolve.comp.FESpace) -> tuple:
        ...
    @typing.overload
    def __init__(self, mesh: ngsolve.comp.Mesh, **kwargs) -> None:
        ...
    @typing.overload
    def __init__(self, fes: ...) -> None:
        ...
    def __setstate__(self, arg0: tuple) -> None:
        ...
class PUFESpace(ngsolve.comp.FESpace):
    """
    
    
    
    Keyword arguments can be:
    
    order: int = 1
      order of finite element space
    complex: bool = False
      Set if FESpace should be complex
    dirichlet: regexpr
      Regular expression string defining the dirichlet boundary.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet = 'top|right'
    dirichlet_bbnd: regexpr
      Regular expression string defining the dirichlet bboundary,
      i.e. points in 2D and edges in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbnd = 'top|right'
    dirichlet_bbbnd: regexpr
      Regular expression string defining the dirichlet bbboundary,
      i.e. points in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbbnd = 'top|right'
    definedon: Region or regexpr
      FESpace is only defined on specific Region, created with mesh.Materials('regexpr')
      or mesh.Boundaries('regexpr'). If given a regexpr, the region is assumed to be
      mesh.Materials('regexpr').
    dim: int = 1
      Create multi dimensional FESpace (i.e. [H1]^3)
    dgjumps: bool = False
      Enable discontinuous space for DG methods, this flag is needed for DG methods,
      since the dofs have a different coupling then and this changes the sparsity
      pattern of matrices.
    autoupdate: bool = False
      Automatically update on a change to the mesh.
    low_order_space: bool = True
      Generate a lowest order space together with the high-order space,
      needed for some preconditioners.
    hoprolongation: bool = False
      Create high order prolongation operators,
      only available for H1 and L2 on simplicial meshes
    order_policy: ORDER_POLICY = ORDER_POLICY.OLDSTYLE
      CONSTANT .. use the same fixed order for all elements,
      NODAL ..... use the same order for nodes of same shape,
      VARIABLE ... use an individual order for each edge, face and cell,
      OLDSTYLE .. as it used to be for the last decade
    print: bool = False
      (historic) print some output into file set by 'SetTestoutFile'
    useshift: bool = True
      shift of basis functins to element center
    usescale: bool = True
      scale element basis functions with diam
    """
    @staticmethod
    def GetDocu() -> ...:
        ...
    @staticmethod
    def __flags_doc__() -> dict:
        ...
    def __getstate__(self: ngsolve.comp.FESpace) -> tuple:
        ...
    def __init__(self, mesh: ngsolve.comp.Mesh, **kwargs) -> None:
        ...
    def __setstate__(self, arg0: tuple) -> None:
        ...
class PrintCF(ngsolve.fem.CoefficientFunction):
    """
    
    CoefficientFunction that writes integration point (in world coords.)
    into a file whenever evaluated at one.
    """
    def __init__(self, filename: str) -> None:
        """
        Constructor of PrintCF (integration point (in world coords.) printing coefficientfunction).
          Argument: filename (string) : name of the file where the values shall be printed
        """
class QTWaveTents1(TrefftzTents):
    def Energy(self, arg0: ngsolve.bla.MatrixD) -> float:
        ...
    def Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def GetInitmesh(self) -> ngsolve.comp.Mesh:
        ...
    def GetOrder(self) -> int:
        ...
    def GetSpaceDim(self) -> int:
        ...
    def GetWave(self, U):
        ...
    def GetWavefront(self) -> ngsolve.bla.MatrixD:
        ...
    def L2Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def LocalDofs(self) -> int:
        ...
    def MakeWavefront(self, arg0: ngsolve.fem.CoefficientFunction, arg1: float) -> ngsolve.bla.MatrixD:
        ...
    def MaxAdiam(self) -> float:
        ...
class QTWaveTents2(TrefftzTents):
    def Energy(self, arg0: ngsolve.bla.MatrixD) -> float:
        ...
    def Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def GetInitmesh(self) -> ngsolve.comp.Mesh:
        ...
    def GetOrder(self) -> int:
        ...
    def GetSpaceDim(self) -> int:
        ...
    def GetWave(self, U):
        ...
    def GetWavefront(self) -> ngsolve.bla.MatrixD:
        ...
    def L2Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def LocalDofs(self) -> int:
        ...
    def MakeWavefront(self, arg0: ngsolve.fem.CoefficientFunction, arg1: float) -> ngsolve.bla.MatrixD:
        ...
    def MaxAdiam(self) -> float:
        ...
class TWaveTents1(TrefftzTents):
    def Energy(self, arg0: ngsolve.bla.MatrixD) -> float:
        ...
    def Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def GetInitmesh(self) -> ngsolve.comp.Mesh:
        ...
    def GetOrder(self) -> int:
        ...
    def GetSpaceDim(self) -> int:
        ...
    def GetWave(self, U):
        ...
    def GetWavefront(self) -> ngsolve.bla.MatrixD:
        ...
    def L2Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def LocalDofs(self) -> int:
        ...
    def MakeWavefront(self, arg0: ngsolve.fem.CoefficientFunction, arg1: float) -> ngsolve.bla.MatrixD:
        ...
    def MaxAdiam(self) -> float:
        ...
class TWaveTents2(TrefftzTents):
    def Energy(self, arg0: ngsolve.bla.MatrixD) -> float:
        ...
    def Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def GetInitmesh(self) -> ngsolve.comp.Mesh:
        ...
    def GetOrder(self) -> int:
        ...
    def GetSpaceDim(self) -> int:
        ...
    def GetWave(self, U):
        ...
    def GetWavefront(self) -> ngsolve.bla.MatrixD:
        ...
    def L2Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def LocalDofs(self) -> int:
        ...
    def MakeWavefront(self, arg0: ngsolve.fem.CoefficientFunction, arg1: float) -> ngsolve.bla.MatrixD:
        ...
    def MaxAdiam(self) -> float:
        ...
class TWaveTents3(TrefftzTents):
    def Energy(self, arg0: ngsolve.bla.MatrixD) -> float:
        ...
    def Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def GetInitmesh(self) -> ngsolve.comp.Mesh:
        ...
    def GetOrder(self) -> int:
        ...
    def GetSpaceDim(self) -> int:
        ...
    def GetWave(self, U):
        ...
    def GetWavefront(self) -> ngsolve.bla.MatrixD:
        ...
    def L2Error(self, arg0: ngsolve.bla.MatrixD, arg1: ngsolve.bla.MatrixD) -> float:
        ...
    def LocalDofs(self) -> int:
        ...
    def MakeWavefront(self, arg0: ngsolve.fem.CoefficientFunction, arg1: float) -> ngsolve.bla.MatrixD:
        ...
    def MaxAdiam(self) -> float:
        ...
class Tent:
    """
    Tent structure
    """
    def MaxSlope(self) -> float:
        ...
    @property
    def els(self) -> pyngcore.pyngcore.Array_I_S:
        ...
    @property
    def internal_facets(self) -> pyngcore.pyngcore.Array_I_S:
        ...
    @property
    def level(self) -> int:
        ...
    @property
    def nbtime(self) -> pyngcore.pyngcore.Array_D_S:
        ...
    @property
    def nbv(self) -> pyngcore.pyngcore.Array_I_S:
        ...
    @property
    def tbot(self) -> float:
        ...
    @property
    def ttop(self) -> float:
        ...
    @property
    def vertex(self) -> int:
        ...
class TentSlab:
    """
    Tent pitched slab in D + 1 time dimensions
    """
    def DrawPitchedTentsVTK(self, vtkfilename: str = 'vtkoutput') -> None:
        """
                 Export the mesh of tents and intermediate advancing fronts
                 to VTK file format for visualization in Paraview.
        """
    def GetNLayers(self) -> int:
        ...
    def GetNTents(self) -> int:
        ...
    def GetSlabHeight(self) -> float:
        ...
    def GetTent(self, arg0: int) -> Tent:
        ...
    def MaxSlope(self) -> float:
        ...
    def PitchTents(self, dt: float, local_ct: bool = False, global_ct: float = 1.0) -> bool:
        """
                 Parameters:--
                   dt: spacetime slab's height in time.
                   local_ct: if True, constrain tent slope by scaling 1/wavespeed
                     with a further local mesh-dependent factor.
                   global_ct: an additional factor to constrain tent slope, which
                     gives flatter tents for smaller values.
        
                 Returns True upon successful tent meshing.
                 -------------
        """
    def SetMaxWavespeed(self, arg0: typing.Any) -> None:
        ...
    def TentData1D(self) -> list:
        ...
    def __init__(self, mesh: ngsolve.comp.Mesh, method: str = 'edge', heapsize: int = 1000000) -> None:
        ...
    @property
    def gradphi(self) -> ngsolve.fem.CoefficientFunction:
        ...
    @property
    def mesh(self) -> ngsolve.comp.Mesh:
        ...
class TrefftzTents:
    def Propagate(self) -> None:
        """
        Solve tent slab
        """
    def SetBoundaryCF(self, arg0: ngsolve.fem.CoefficientFunction) -> None:
        """
        Set boundary condition
        """
    def SetInitial(self, arg0: ngsolve.fem.CoefficientFunction) -> None:
        """
        Set initial condition
        """
class VectorL2EmbTrefftzFESpace(ngsolve.comp.FESpace):
    """
    A vector-valued L2-conforming finite element space.
    
    The Vector-L2 finite element space is a product-space of L2 spaces,
    where the number of components coincides with the mesh dimension.
    
    It is implemented by means of a CompoundFESpace, as one could do it at the
    user-level. Additionally, some operators are added for convenience and performance:
    One can evaluate the vector-valued function, and one can take the gradient.
    
    Keyword arguments can be:
    
    order: int = 1
      order of finite element space
    complex: bool = False
      Set if FESpace should be complex
    dirichlet: regexpr
      Regular expression string defining the dirichlet boundary.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet = 'top|right'
    dirichlet_bbnd: regexpr
      Regular expression string defining the dirichlet bboundary,
      i.e. points in 2D and edges in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbnd = 'top|right'
    dirichlet_bbbnd: regexpr
      Regular expression string defining the dirichlet bbboundary,
      i.e. points in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbbnd = 'top|right'
    definedon: Region or regexpr
      FESpace is only defined on specific Region, created with mesh.Materials('regexpr')
      or mesh.Boundaries('regexpr'). If given a regexpr, the region is assumed to be
      mesh.Materials('regexpr').
    dim: int = 1
      Create multi dimensional FESpace (i.e. [H1]^3)
    dgjumps: bool = False
      Enable discontinuous space for DG methods, this flag is needed for DG methods,
      since the dofs have a different coupling then and this changes the sparsity
      pattern of matrices.
    autoupdate: bool = False
      Automatically update on a change to the mesh.
    low_order_space: bool = True
      Generate a lowest order space together with the high-order space,
      needed for some preconditioners.
    hoprolongation: bool = False
      Create high order prolongation operators,
      only available for H1 and L2 on simplicial meshes
    order_policy: ORDER_POLICY = ORDER_POLICY.OLDSTYLE
      CONSTANT .. use the same fixed order for all elements,
      NODAL ..... use the same order for nodes of same shape,
      VARIABLE ... use an individual order for each edge, face and cell,
      OLDSTYLE .. as it used to be for the last decade
    print: bool = False
      (historic) print some output into file set by 'SetTestoutFile'
    piola: bool = False
      Use Piola transform to map to physical element
      allows to use the div-differential operator.
    covariant: bool = False
      Use the covariant transform to map to physical element
      allows to use the curl-differential operator.
    all_dofs_together: bool = True
      dofs within one scalar component are together.
    hide_all_dofs: bool = False
      all dofs are condensed without a global dofnr
    lowest_order_wb: bool = False
      Keep lowest order dof in WIRE_BASKET
    tp: bool = False
      Use sum-factorization for evaluation
    """
    @staticmethod
    def __flags_doc__() -> dict:
        ...
    def Embed(self, arg0: ngsolve.comp.GridFunction) -> ngsolve.comp.GridFunction:
        ...
    def GetEmbedding(self) -> BaseMatrix:
        ...
    @typing.overload
    def SetOp(self, bf: ngsolve.comp.SumOfIntegrals, lf: ngsolve.comp.SumOfIntegrals = None, eps: float = 0, test_fes: ngsolve.comp.FESpace = None, tndof: int = 0) -> ngsolve.la.BaseVector:
        """
        Sets the operators for the embedded Trefftz method.
        """
    @typing.overload
    def SetOp(self, op: ngsolve.comp.SumOfIntegrals | None, cop_lhs: ngsolve.comp.SumOfIntegrals | None, cop_rhs: ngsolve.comp.SumOfIntegrals | None, fes_conformity: ngsolve.comp.FESpace, fes_test: ngsolve.comp.FESpace = None, linear_form: ngsolve.comp.SumOfIntegrals = None, ndof_trefftz: int | float = 0) -> ngsolve.la.BaseVector:
        """
                    Sets the operators for the conforming Trefftz method.
        
                    :param op: the differential operation. Can be None
                    :param cop_lhs: left hand side of the conformity operation
                    :param cop_rhs: right hand side of the conformity operation
                    :param fes_conformity: finite element space of the conformity operation
                    :param fes_test: test finite element space for `op`. Can be None
                    :param linear_form: right hand side of the var. formulation. Can be None
                    :param ndof_trefftz: number of degrees of freedom per element
                        in the Trefftz finite element space on `fes`, generated by `op`
                        (i.e. the local dimension of the kernel of `op` on one element)
        
                    :return: the particular solution vector.
        """
    def __getstate__(self: ngsolve.comp.FESpace) -> tuple:
        ...
    @typing.overload
    def __init__(self, mesh: ngsolve.comp.Mesh, **kwargs) -> None:
        ...
    @typing.overload
    def __init__(self, fes: ngsolve.comp.VectorL2) -> None:
        ...
    def __setstate__(self, arg0: tuple) -> None:
        ...
class WeightedRadiusFunction(ngsolve.fem.CoefficientFunction):
    def __init__(self, mesh: ngsolve.comp.Mesh, CoefficientFunction: ngsolve.fem.CoefficientFunction) -> None:
        ...
class monomialfespace(ngsolve.comp.FESpace):
    """
    
    
    
    Keyword arguments can be:
    
    order: int = 1
      order of finite element space
    complex: bool = False
      Set if FESpace should be complex
    dirichlet: regexpr
      Regular expression string defining the dirichlet boundary.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet = 'top|right'
    dirichlet_bbnd: regexpr
      Regular expression string defining the dirichlet bboundary,
      i.e. points in 2D and edges in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbnd = 'top|right'
    dirichlet_bbbnd: regexpr
      Regular expression string defining the dirichlet bbboundary,
      i.e. points in 3D.
      More than one boundary can be combined by the | operator,
      i.e.: dirichlet_bbbnd = 'top|right'
    definedon: Region or regexpr
      FESpace is only defined on specific Region, created with mesh.Materials('regexpr')
      or mesh.Boundaries('regexpr'). If given a regexpr, the region is assumed to be
      mesh.Materials('regexpr').
    dim: int = 1
      Create multi dimensional FESpace (i.e. [H1]^3)
    dgjumps: bool = False
      Enable discontinuous space for DG methods, this flag is needed for DG methods,
      since the dofs have a different coupling then and this changes the sparsity
      pattern of matrices.
    autoupdate: bool = False
      Automatically update on a change to the mesh.
    low_order_space: bool = True
      Generate a lowest order space together with the high-order space,
      needed for some preconditioners.
    hoprolongation: bool = False
      Create high order prolongation operators,
      only available for H1 and L2 on simplicial meshes
    order_policy: ORDER_POLICY = ORDER_POLICY.OLDSTYLE
      CONSTANT .. use the same fixed order for all elements,
      NODAL ..... use the same order for nodes of same shape,
      VARIABLE ... use an individual order for each edge, face and cell,
      OLDSTYLE .. as it used to be for the last decade
    print: bool = False
      (historic) print some output into file set by 'SetTestoutFile'
    useshift: bool = True
      shift of basis functins to element center
    usescale: bool = True
      scale element basis functions with diam
    """
    @staticmethod
    def GetDocu() -> ...:
        ...
    @staticmethod
    def __flags_doc__() -> dict:
        ...
    def SetCoeff(self, arg0: ngsolve.fem.CoefficientFunction) -> None:
        ...
    def __getstate__(self: ngsolve.comp.FESpace) -> tuple:
        ...
    def __init__(self, mesh: ngsolve.comp.Mesh, **kwargs) -> None:
        ...
    def __setstate__(self, arg0: tuple) -> None:
        ...
class trefftzfespace(ngsolve.comp.FESpace):
    """
    Trefftz space for different PDEs. Use kwarg 'eq' to choose the PDE, currently implemented are:
     - laplace - for Laplace equation
     - qtelliptic - for the quasi-Trefftz space for an elliptic problem
     - wave - for the second order acoustic wave equation
     - qtwave - for the quasi-Trefftz space
     - fowave - for the first order acoustic wave equation, returns TnT (sigv,tauw)
     - foqtwave - for the quasi-Trefftz space 
     - helmholtz - planewaves for the helmholtz equation
     - helmholtzconj - returns the complex conjungate of the planewaves 
    
    
    
    Keyword arguments can be:
    
    eq: string
      Choose type of Trefftz functions.
    order: int = 1
      Order of finite element space
    dgjumps: bool = True
      Enable discontinuous space for DG methods, this flag is always True for trefftzfespace.
    complex: bool = False
      Set if FESpace should be complex
    useshift: bool = True
      shift of basis functins to element center
    usescale: bool = True
      scale element basis functions with diam
    """
    @staticmethod
    def GetDocu() -> ...:
        ...
    @staticmethod
    def __flags_doc__() -> dict:
        ...
    def GetEWSolution(self, acoeffF: ngsolve.fem.CoefficientFunction) -> ngsolve.comp.GridFunction:
        """
                        Compute a element-wise particular solution for given right hand side.
        
                        Parameters
                        ----------
                        coeffF : CoefficientFunction
                            Right hand side
        """
    @typing.overload
    def SetCoeff(self, coeff_const: float) -> None:
        ...
    @typing.overload
    def SetCoeff(self, acoeffA: ngsolve.fem.CoefficientFunction, acoeffB: ngsolve.fem.CoefficientFunction = None, acoeffC: ngsolve.fem.CoefficientFunction = None) -> None:
        """
                        Set coefficient of Trefftz space.
        
                        For an elliptic problem, the coefficients are given by
                        - div(coeffA*grad(u)) + coeffB*grad(u) + coeffC u = 0
        
                        For the first order wave equation, the coefficients are given by
                        grad(v) + coeffB dt sigma = 0
                        div(sigma) + 1/coeffA**2 dt v = 0
        
                        For the second order wave equation, the coefficients are given by
                        - div(1/coeffB grad(u)) + 1/coeffA**2 dtt u = 0
        
                        Parameters
                        ----------
                        coeffA : CoefficientFunction
                            Coefficient A
                        coeffB : CoefficientFunction
                            Coefficient B
                        coeffC : CoefficientFunction
                            Coefficient C
        """
    def __getstate__(self: ngsolve.comp.FESpace) -> tuple:
        ...
    def __init__(self, mesh: ngsolve.comp.Mesh, **kwargs) -> None:
        ...
    def __setstate__(self, arg0: tuple) -> None:
        ...
def ClipCoefficientFunction(arg0: ngsolve.fem.CoefficientFunction, arg1: int, arg2: float) -> ngsolve.fem.CoefficientFunction:
    ...
def CondenseDG(mat: BaseMatrix, vec: ngsolve.la.BaseVector, ncondense: ngsolve.comp.FESpace) -> BaseMatrix:
    """
          hello
    """
def EmbeddedTrefftzFES(fes: ngsolve.comp.FESpace) -> ngsolve.comp.FESpace:
    """
                            Given a FESpace this wrapper produces a Trefftz FESpace using local projections, following the Embedded Trefftz-DG methodology. Use EmbTrefftzFES.SetOp() to set the operator used to construct the embedding.
    
                            :param fes: FESpace to be wrapped.
    
                            :return: EmbTrefftzFES
    """
def FFacetBFI(form: ngsolve.fem.CoefficientFunction, VOL_or_BND: ngsolve.comp.VorB = ..., element_boundary: bool = False, skeleton: bool = True, definedon: ngsolve.comp.Region | list | None = None, intrule: ngsolve.fem.IntegrationRule = ..., bonus_intorder: int = 0, definedonelements: pyngcore.pyngcore.BitArray = None, simd_evaluate: bool = False, element_vb: ngsolve.comp.VorB = ..., geom_free: bool = False, deformation: ngsolve.comp.GridFunction = None) -> ngsolve.fem.BFI:
    """
    A symbolic bilinear form integrator, operating on facet intersections.
    
    Parameters:
    
    VOL_or_BND : ngsolve.comp.VorB
      input VOL, BND
    
    skeleton : bool
      must be True
    """
def FFacetLFI(form: ngsolve.fem.CoefficientFunction, VOL_or_BND: ngsolve.comp.VorB = ..., element_boundary: bool = False, skeleton: bool = True, definedon: ngsolve.comp.Region | list | None = None, intrule: ngsolve.fem.IntegrationRule = ..., bonus_intorder: int = 0, definedonelements: pyngcore.pyngcore.BitArray = None, simd_evaluate: bool = False, element_vb: ngsolve.comp.VorB = ..., deformation: ngsolve.comp.GridFunction = None) -> ngsolve.fem.LFI:
    """
    A symbolic linear form integrator, operating on facet intersections.
    
    Parameters:
    
    VOL_or_BND : ngsolve.comp.VorB
      input VOL, BND
    
    skeleton : bool
      must be True
    """
def GetWave(self, U):
    ...
def Mesh1dTents(arg0: TentSlab) -> ngsolve.comp.Mesh:
    ...
def SpaceTimeDG_FFacetBFI(mesh: ngsolve.comp.Mesh, coef_c: ngsolve.fem.CoefficientFunction, coef_sig: ngsolve.fem.CoefficientFunction, VOL_or_BND: ngsolve.comp.VorB) -> ngsolve.fem.BFI:
    ...
def SpaceTimeDG_FFacetLFI(mesh: ngsolve.comp.Mesh, gfuh: ngsolve.fem.CoefficientFunction, gfduh: ngsolve.fem.CoefficientFunction, coef_c: ngsolve.fem.CoefficientFunction, coef_sig: ngsolve.fem.CoefficientFunction, VOL_or_BND: ngsolve.comp.VorB) -> ngsolve.fem.LFI:
    ...
def TWave(order: int, tps: TentSlab, wavespeedcf: ngsolve.fem.CoefficientFunction, BBcf: ngsolve.fem.CoefficientFunction = None) -> TrefftzTents:
    """
                    Create solver for acoustiv wave equation on tent-pitched mesh.
    
                    :param order: Polynomial order of the Trefftz space.
                    :param tps: Tent-pitched slab.
                    :param wavespeedcf: PDE Coefficient
                    :param BB: PDE Coefficient
    """
@typing.overload
def TrefftzEmbedding(bf: ngsolve.comp.SumOfIntegrals, fes: ngsolve.comp.FESpace, lf: ngsolve.comp.SumOfIntegrals, eps: float = 0, test_fes: ngsolve.comp.FESpace = None, tndof: int = 0, getrange: bool = False, stats_dict: dict | None = None) -> tuple[BaseMatrix, ngsolve.la.BaseVector]:
    """
                    Computes the Trefftz embedding and particular solution.
    
                    :param bf: operator for which the Trefftz embedding is computed.
                    :param fes: DG finite element space of the weak formulation.
                    :param lf: Rhs used to compute the particular solution.
                    :param eps: Threshold for singular values to be considered zero, defaults to 0
                    :param test_fes: Used if test space differs from trial space, defaults to None
                    :param tndof: If known, local ndofs of the Trefftz space, else eps and/or test_fes are used to find the dimension
                    :param getrange: If True, extract the range instead of the kernel
                    :param stats_dict: Pass a dictionary to fill it with stats on the singular values.
    
                    :return: [Trefftz embedding, particular solution]
    """
@typing.overload
def TrefftzEmbedding(bf: ngsolve.comp.SumOfIntegrals, fes: ngsolve.comp.FESpace, eps: float = 0, test_fes: ngsolve.comp.FESpace = None, tndof: int = 0, getrange: bool = False, stats_dict: dict | None = None) -> BaseMatrix:
    """
                    Used without the parameter lf as input the function only returns the Trefftz embedding.
    
                    :return: Trefftz embedding
    """
@typing.overload
def TrefftzEmbedding(op: ngsolve.comp.SumOfIntegrals | None, fes: ngsolve.comp.FESpace, cop_lhs: ngsolve.comp.SumOfIntegrals, cop_rhs: ngsolve.comp.SumOfIntegrals, fes_conformity: ngsolve.comp.FESpace, ndof_trefftz: int | float = None, fes_test: ngsolve.comp.FESpace = None) -> BaseMatrix:
    """
                    creates an embedding matrix P for the given operations `op`,
                    `cop_lhs`, `cop_rhs`.
                    The embedding is subject to the conformitys specified in
                    `cop_lhs` and `cop_rhs`.
    
                     :param op: the differential operation. Can be None
                     :param fes: the finite element space of `op`
                     :param cop_lhs: left hand side of the conformity operation
                     :param cop_rhs: right hand side of the conformity operation
                     :param fes_conformity: finite element space of the conformity operation
                     :param ndof_trefftz: number of degrees of freedom per element
                         in the Trefftz finite element space on `fes`, generated by `op`
                         (i.e. the local dimension of the kernel of `op` on one element)
    
                     :return: P, the embedding matrix.
    """
@typing.overload
def TrefftzEmbedding(op: ngsolve.comp.SumOfIntegrals | None, fes: ngsolve.comp.FESpace, cop_lhs: ngsolve.comp.SumOfIntegrals, cop_rhs: ngsolve.comp.SumOfIntegrals, fes_conformity: ngsolve.comp.FESpace, linear_form: ngsolve.comp.SumOfIntegrals, ndof_trefftz: int | float = None, fes_test: ngsolve.comp.FESpace = None) -> tuple[BaseMatrix, ngsolve.la.BaseVector]:
    """
                    creates an embedding matrix P for the given operations `op`,
                    `cop_lhs`, `cop_rhs`.
                    The embedding is subject to the conformitys specified in
                    `cop_lhs` and `cop_rhs`.
                    Also generates a particular solution `u_lf`.
    
                     :param op: the differential operation. Can be None
                     :param fes: the finite element space of `op`
                     :param cop_lhs: left hand side of the conformity operation
                     :param cop_rhs: right hand side of the conformity operation
                     :param fes_conformity: finite element space of the conformity operation
                     :param linear_form: right hand side of the var. formulation
                     :param ndof_trefftz: number of degrees of freedom per element
                         in the Trefftz finite element space on `fes`, generated by `op`
                         (i.e. the local dimension of the kernel of `op` on one element)
    
                     :return: P, the embedding matrix.
    """
dbox: BoxDifferentialSymbol  # value = <ngstrefftz.BoxDifferentialSymbol object>
ngstrefftz = 
